###############################################################################
###############################################################################
###                                                                         ###
###               ONE-DIMENSIONAL HEAT and MASS TRANSFER MODEL              ###
###                  for VERTICAL GREENERY SYSTEMS                          ###
###                                                                         ###
###                             VGheat1D 1.0                                ###
###                                                                         ###
###                 doi:  http://dx.doi.org/10.14279/depositonce-10512      ###
###                                                                         ###
###       Author: Karin Hoffmann                                            ###
###               Chair of Ecohydrology and Landscape Evaluation            ###
###               Technische Universitt Berlin                             ###
###                                                                         ###
###       Support:Dr. Tomaz Suklje                                          ###
###               Sensedge d.o.o., Podbreznik 15, 8000 Novo mesto, Slovenia ###
###                                                                         ###  
###      Based on: T. Suklje, S. Medved and C. Arkar,                       ###
###               On detailed thermal response modeling of vertical         ###
###               greenery systems as cooling measure for buildings         ### 
###               and cities in summer conditions,                          ###
###               Energy, 115 (2016) 1055-1068                              ###
###                                                                         ###    
###                                                                         ###
###      Berlin, September 2020     contact: vgheat1D@gmail.com             ###
###                                                                         ###
###############################################################################

#1. set working directory, load libraries, and source functions
setwd('C:/Users/X250/Documents/vgheat_v1')

# What is the name of the scenario? That's the name by which the output will be saved
scen_name <- "new_scenario"

#What is the name of the building? B01 to B05 and LJ01 to LJ05 or LJtestcell
buildg_name <- "LJtestcell"

datapath <- "./data/meteo_sample.dat"
ode_path <- "./func/t_func_ODE_LJtestcell.R"

# What is the temporal resolution of the meteorological input file? (set to 1 for hourly, 2 for minutes)
temp_res <- 1

#Is the wall greened (1) or not greened (0)?

greening <- 1

# What is the number of greenery layers ? (1 or 2)

green_layer <- 1

#Is there a reduced evapotranspiration? If not, leave value at 1; if raised by 30%, set to 1.3, if lowered by 20% set to 0.8 etc. 

ET_fac <- 1

# locked indoor temperature

Tair_i<- 25        # Temperature of the interior air; if it is a time series, set it in line 178  

# to which temporal res. should the output be summarized? n_n = 1 for seconds, n_n = 60 for minutes, n_n = 3600 for hours etc. 

n_n <- 60

# Which time range do you want to calculate? ! 1<= start_hour <= 24
start_month <- 1
start_day <- 1
start_hour <- 1

end_month <- 1
end_day <- 1
end_hour <- 5

# wall material parameters:

#How many material layers does the wall have (set 3 OR 4 OR 5)
#LJ test cell

lay_num <- 4 # number between 1 and 10
nodes_num <- 23


lambda_wall1 = 1.1      # heat conductivity wall [W/(m*K)]
rho_w1 = 1800           # density wall [kg/m^3]
cp_wall1 <- 880         # specific heat capacity wall [J/(kg*K)]
d1 = 0.008              # thickness wall; assumption: VGS as water layer [m]

lambda_wall2 = 0.034    # heat conductivity wall [W/(m*K)]
rho_w2 = 30             # density wall [kg/m^3]
cp_wall2 <- 1200        # specific heat capacity wall [J/(kg*K)]
d2 = 0.150               # thickness wall; assumption: VGS as water layer [m]

lambda_wall3 = 0.13     # heat conductivity wall [W/(m*K)]
rho_w3 = 450            # density wall [kg/m^3]
cp_wall3 <- 1000        # specific heat capacity wall [J/(kg*K)]
d3 = 0.075             # thickness wall; assumption: VGS as water layer [m]

lambda_wall4 = 1.1     # heat conductivity wall [W/(m*K)]
rho_w4 = 1800            # density wall [kg/m^3]
cp_wall4 <- 880        # specific heat capacity wall [J/(kg*K)]
d4 = 0.004             # thickness wall; assumption: VGS as water layer [m]

# thickness wall; assumption: VGS as water layer [m]


#######################################################
#######################################################


data <- read.table(datapath, skip = 3, sep = "\t", dec =".", header =TRUE)

start <- which((data$m == start_month & data$dm == start_day & data$h == start_hour ))
end <- which((data$m == end_month & data$dm == end_day & data$h == end_hour))

data <- data[start:end,]



#install.packages("zoo")
#install.packages("tidyr")

library(tidyr)
library(zoo)
library(deSolve)

################################################



source("./func/delta.R")
source("./func/es.R")
source("./func/ea.R")
source("./func/Rl_down.R")
source("./func/Rs_sol_VGS.R")
source("./func/Rs_sol_se.R")
source("./func/interpol_func.R")

############################
#2. choose meteorological input data and select time span 




##################################  
#3. set input parameters 

# additional wall  parameters:

  beta <- 90              # angle between wall and ground
  emis_lw_se <- 0.98      # emissivity of the wall surface as suggested by Suklje 2016
  alpha_sol_se <- 0.75    # solar absorptivity of the exterior wall surface [-]
  h_wall <- 2.45          # height wall [m] 


#plant parameters:
if (green_layer == 1){
  
  hc <- 0.3               # crop thickness [m] (one-layered VGS -> Suklje 2016)
  d_vgs = 0.002           # thickness VGS; assumption: VGS as water layer [m] Suklje2016: water layer of 4mm
  LAI <- 6.1              # Leaf Area Index [-] (experimental data from Suklje 2016 -> 6.1+- 0.5)
  
} else {
  
  hc <- 0.6               # crop thickness [m] (two-layered VGS -> Suklje 2016)
  d_vgs = 0.004           # thickness VGS; assumption: VGS as water layer [m] Suklje2016: water layer of 4mm
  LAI <- 7.2              # Leaf Area Index [-] (experimental data from Suklje 2016 -> 7.2+- 0.6)
  
}
  
  r_l <- 100              # stomatal resistance[s/m]
  emis_lw_VGS <- 0.983    # as suggested by Suklje 2016
  
  if (greening == 1){
    if (green_layer == 1){
      
      tau <- 0.17             # transmissivity of the VGS [-] (one-layered) (Suklje 2016: 0.17+-0.02)
      FF <- 0.042             # view factor,as suggested by Suklje 2016 for a one -layered VGS
      
    } else {
      
      tau <- 0.09             # transmissivity of the VGS [-] (two-layered) (Suklje 2016: 0.09+-0.02)
      FF <- 0.005             # view factor,as suggested by Suklje 2016 for a one -layered VGS
      
    }
  
  } else {
    
    tau <- 1              # transmissivity if there's no greening
    FF <- 1
    nodes_num <- nodes_num - 2
    }
  
  alpha_sol_VGS <- 0.768  # solar absorptivity of the VGS [-]
  rho_vgs = 1000          # density VGS; assumption: VGS as water layer [kg/m^3]
  cp_vgs = 4186           # specific heat capacity VGS; assumption: VGS as water layer [J/(kg*K)]
  
  
  #meteorological parameters:
  u_z <- data[,12]        # wind speed [m/s]
  u_z[u_z < 0.1] <- 0.1
  Tair <- data[,8]        # air temperature [C]
  relH <- data[,11]       # relative Humidity [%]
  Rglob <- data[,6]       # Global rad received by an inclined surface [W/m] (Gk)
  Tdew <- data[,9]        # dew point temperature [C]
  cl_cov <- data[,14]     # cloud coverage [octal units]
  #Tair_i <- 
  
  p_atm <- 97.7          # Atmospherc pressure [kPa]
  h_conv_si <- 7.7       # heat transfer coefficient inner wall surface [W/(mK])] according to EN ISO 13786 (includes radiation and convention)
  
  if (green_layer == 1){
    
    z_m <- 2                # height of wind measurements [m]
    z_h <- 2                # height of humidity measurements [m]
    
  } else if(green_layer == 2){
    
    z_m <- 1.7               # height of wind measurements [m]
    z_h <- 1.7                # height of humidity measurements [m]
    
    
  } else {
    print("Number of greening layers not recognized")
  }
 
  gamma <- 0.000665 * p_atm  # pyschrometric constant acc. to FAO (online); [kPa/ C]
  rho_a <- 1.2041            # air density [kg/m^3]
  cp_air <- 1005             # specific heat capacity of air [J/(kg*K)]
  sigma <-  5.67*10^(-8)     # Stefan-Boltzmann-constant [W/(m*K^4)]
  kin_visc_a <- 0.0000155    # kinematic viscosity of air [m/s] according to stanghellini1993
  u_a <- 0.2                 # wind velocity in the air gap [m/s]
  d_ag <- 0.08               # thickness of the air gap [m] (Suklje 2016)
  lambda_air <- 0.02435      # thermal conductivity of air [w/(m*K)]
  
 
  #r_lat <- 2490000        # evaporation enthalpy of water [J/kg]

  
##########################################
#4. calculate missing radiation data
  ntime <- NROW(data[,1])
  q_IR_down <- Rl_down_func(Tair, Tdew, cl_cov )
  
  
  q_sol_VGS <- Rs_sol_VGS_func(tau, alpha_sol_VGS, Rglob)
  
  q_sol_se <- Rs_sol_se_func(tau, alpha_sol_se, Rglob)
  
#########################################  
#5. prepare lat heat calculation
  
  # in calculating r_a, exclude values where wind speed == 0
  u_z_n0 <- u_z
  u_z_n0[which(u_z_n0 == 0)] <- NA
  
  ntime <- NROW(data[,1]) #  2nd order input parameters need to have the length of 
  #input time series in seconds resolution!
  
  z_om     <- 0.123 * hc
  z_oh     <- 0.1 * z_om
  dh       <- (2/3)*hc
  LAIact   <- 0.5 * LAI
  r_s      <- r_l / LAIact
  
  r_a <- rep(NA, ntime);
  delta <- rep(NA, ntime);
  es <- rep(NA, ntime);
  ea <- rep(NA, ntime);
  
  for(i in 1:ntime){   
    
    
    delta[i]    <- Delta(Tair[i])
    es[i]       <- satp(Tair[i])
    ea[i]       <- VapPres(relH[i], es[i])
    r_a[i]      <- (log((z_m-dh)/z_om)*log((z_h-dh)/z_oh))/((0.41^2)*u_z_n0[i])
    
  } 
    
  #6. expand and interpolate all input data taken from measured data (LJ test cell) file
  
  
  #define input for interpolation
  if (temp_res == 1){
    how_many_sec <- 3600 
  } else if (temp_res == 2){
    
    how_many_sec <- 60
  } else {
    
    print("temporal resoltion of input data not valid")
    
    break
    
  }
  
  
  u_z       <- interpol_sec(u_z)         # wind speed [m/s]
  Tair      <- interpol_sec(Tair)         # air temperature [C]
  relH      <- interpol_sec(relH)         # relative Humidity [%]
  Rglob     <- interpol_sec(Rglob)       # Global rad received by an inclined surface [W/m] (Gk)
  q_IR_down <- interpol_sec(q_IR_down)
  Tair_i    <- interpol_sec(Tair_i)
  Tdew      <- interpol_sec(Tdew)         # dew point temperature [C]
  cl_cov    <- interpol_sec(cl_cov)   
  q_sol_VGS <- interpol_sec(q_sol_VGS)
  q_sol_se  <- interpol_sec(q_sol_se)
  
                        
  #set negative values to zero, as these are artefacts from the interpolation and are incorrect per definition
  for(i in 1:length(u_z)){
    if(u_z[i] < 0){
      u_z[i] <- 0
      } 
  }
  


  #call ODE caller that calls ODE ;)
  source("./func/func_callODE.R")
